=== modified file 'NEWS'
--- NEWS	2005-05-02 04:35:02 +0000
+++ NEWS	2005-05-03 07:48:16 +0000
@@ -19,6 +19,11 @@
 
     * Included ElementTree library upgraded to 1.2.6 by Fredrik Lundh.
 
+    * Refactor command functions into Command objects based on
+      mango-sorbet by Scott James Remnant.
+
+    * Better help messages for many commands.
+
 
 bzr-0.0.4  2005-04-22
 

=== modified file 'TODO'
--- TODO	2005-05-03 01:43:08 +0000
+++ TODO	2005-05-03 07:48:16 +0000
@@ -51,10 +51,10 @@
   commands where this shouldn't be done, such as 'bzr ignore', because
   we want to accept globs.
 
-* 'bzr ignore' command that just adds a line to the .bzrignore file
-  and makes it versioned.
+* ``bzr ignore`` command that just adds a line to the ``.bzrignore`` file
+  and makes it versioned.  Fix this to break symlinks.
 
-* 'bzr help commands' should give a one-line summary of each command.
+* ``bzr help commands`` should give a one-line summary of each command.
 
 * Any useful sanity checks in 'bzr ignore'?  Perhaps give a warning if
   they try to add a single file which is already versioned, or if they
@@ -67,6 +67,17 @@
 
 * Separate read and write version checks?
 
+* ``bzr status FILE...``
+
+* Check all commands have decent help.
+
+* Autogenerate argument/option help.
+
+* ``bzr inventory -r REV`` and perhaps unify this with ``bzr ls``,
+  giving options to display ids, types, etc.
+
+* Atomic file class that renames into place when it's closed.
+
 
 Medium things
 -------------
@@ -93,7 +104,7 @@
 * Turn on stat cache code, and add optimization about avoiding
   dangerous cache entries.
 
-* mv command?
+* ``bzr mv`` that does either rename or move as in Unix.
 
 * More efficient diff of only selected files.
 
@@ -162,6 +173,14 @@
 
   Eventually symlinks should be versioned.
 
+* Allow init in a subdirectory to create a nested repository, but only
+  if the subdirectory is not already versioned.   Perhaps also require
+  a ``--nested`` to protect against confusion.
+
+* Branch names? 
+
+
+
 
 Large things
 ------------
@@ -207,3 +226,4 @@
   exports first as a simpler case because then we don't need to deal
   with removing the tags on the way back in.
 
+* ``bzr find``

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2005-05-02 04:35:02 +0000
+++ bzrlib/commands.py	2005-05-03 07:48:16 +0000
@@ -1,6 +1,4 @@
-# Copyright (C) 2004, 2005 by Martin Pool
-# Copyright (C) 2005 by Canonical Ltd
-
+# Copyright (C) 2004, 2005 by Canonical Ltd
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -50,7 +48,7 @@
       Show changes from last revision to working copy.
   bzr commit -m 'MESSAGE'
       Store current state as new revision.
-  bzr export REVNO DESTINATION
+  bzr export [-r REVNO] DESTINATION
       Export the branch state at a previous version.
   bzr status
       Show summary of pending changes.
@@ -71,12 +69,11 @@
 from pprint import pprint
 from stat import *
 from glob import glob
-from inspect import getdoc
 
 import bzrlib
 from bzrlib.store import ImmutableStore
 from bzrlib.trace import mutter, note, log_error
-from bzrlib.errors import bailout, BzrError, BzrCheckError
+from bzrlib.errors import bailout, BzrError, BzrCheckError, BzrCommandError
 from bzrlib.osutils import quotefn, pumpfile, isdir, isfile
 from bzrlib.tree import RevisionTree, EmptyTree, WorkingTree, Tree
 from bzrlib.revision import Revision
@@ -91,27 +88,7 @@
 EMPTY = 'empty'
 
 
-## TODO: Perhaps a different version of inventory commands that
-## returns iterators...
-
-## TODO: Perhaps an AtomicFile class that writes to a temporary file and then renames.
-
-## TODO: Some kind of locking on branches.  Perhaps there should be a
-## parameter to the branch object saying whether we want a read or
-## write lock; release it from destructor.  Perhaps don't even need a
-## read lock to look at immutable objects?
-
-## TODO: Perhaps make UUIDs predictable in test mode to make it easier
-## to compare output?
-
-## TODO: Some kind of global code to generate the right Branch object
-## to work on.  Almost, but not quite all, commands need one, and it
-## can be taken either from their parameters or their working
-## directory.
-
-
-
-cmd_aliases = {
+CMD_ALIASES = {
     '?':         'help',
     'ci':        'commit',
     'checkin':   'commit',
@@ -121,57 +98,103 @@
     }
 
 
-def get_cmd_handler(cmd):
+def get_cmd_class(cmd):
     cmd = str(cmd)
     
-    cmd = cmd_aliases.get(cmd, cmd)
+    cmd = CMD_ALIASES.get(cmd, cmd)
     
     try:
-        cmd_handler = globals()['cmd_' + cmd.replace('-', '_')]
+        cmd_class = globals()['cmd_' + cmd.replace('-', '_')]
     except KeyError:
         raise BzrError("unknown command %r" % cmd)
 
-    return cmd, cmd_handler
-
-
-
-def cmd_status(all=False):
+    return cmd, cmd_class
+
+
+
+class Command:
+    """Base class for commands.
+
+    The docstring for an actual command should give a single-line
+    summary, then a complete description of the command.  A grammar
+    description will be inserted.
+
+    takes_args
+        List of argument forms, marked with whether they are optional,
+        repeated, etc.
+
+    takes_options
+        List of options that may be given for this command.
+
+    hidden
+        If true, this command isn't advertised.
+    """
+    aliases = []
+    
+    takes_args = []
+    takes_options = []
+
+    hidden = False
+    
+    def __init__(self, options, arguments):
+        """Construct and run the command.
+
+        Sets self.status to the return value of run()."""
+        assert isinstance(options, dict)
+        assert isinstance(arguments, dict)
+        cmdargs = options.copy()
+        cmdargs.update(arguments)
+        assert self.__doc__ != Command.__doc__, \
+               ("No help message set for %r" % self)
+        self.status = self.run(**cmdargs)
+
+    
+    def run(self):
+        """Override this in sub-classes.
+
+        This is invoked with the options and arguments bound to
+        keyword parameters.
+
+        Return True if the command was successful, False if not.
+        """
+        return True
+
+
+
+class cmd_status(Command):
     """Display status summary.
 
     For each file there is a single line giving its file state and name.
     The name is that in the current revision unless it is deleted or
     missing, in which case the old name is shown.
     """
-    #import bzrlib.status
-    #bzrlib.status.tree_status(Branch('.'))
-    Branch('.').show_status(show_all=all)
-
-
-
-######################################################################
-# examining history
-def cmd_get_revision(revision_id):
-    Branch('.').get_revision(revision_id).write_xml(sys.stdout)
-
-
-def cmd_get_file_text(text_id):
-    """Get contents of a file by hash."""
-    sf = Branch('.').text_store[text_id]
-    pumpfile(sf, sys.stdout)
-
-
-
-######################################################################
-# commands
-    
-
-def cmd_revno():
-    """Show number of revisions on this branch"""
-    print Branch('.').revno()
-    
-
-    
-def cmd_add(file_list, verbose=False):
+    takes_options = ['all']
+    
+    def run(self, all=False):
+        #import bzrlib.status
+        #bzrlib.status.tree_status(Branch('.'))
+        Branch('.').show_status(show_all=all)
+
+
+class cmd_cat_revision(Command):
+    """Write out metadata for a revision."""
+
+    hidden = True
+    takes_args = ['revision_id']
+    
+    def run(self, revision_id):
+        Branch('.').get_revision(revision_id).write_xml(sys.stdout)
+
+
+class cmd_revno(Command):
+    """Show current revision number.
+
+    This is equal to the number of revisions on this branch."""
+    def run(self):
+        print Branch('.').revno()
+
+    
+class cmd_add(Command):
     """Add specified files or directories.
 
     In non-recursive mode, all the named items are added, regardless
@@ -192,45 +215,55 @@
     TODO: Perhaps adding a file whose directly is not versioned should
     recursively add that parent, rather than giving an error?
     """
-    bzrlib.add.smart_add(file_list, verbose)
-    
-
-def cmd_relpath(filename):
-    """Show path of file relative to root"""
-    print Branch(filename).relpath(filename)
-
-
-
-def cmd_inventory(revision=None):
-    """Show inventory of the current working copy."""
-    ## TODO: Also optionally show a previous inventory
-    ## TODO: Format options
-    b = Branch('.')
-    if revision == None:
-        inv = b.read_working_inventory()
-    else:
-        inv = b.get_revision_inventory(b.lookup_revision(revision))
-        
-    for path, entry in inv.iter_entries():
-        print '%-50s %s' % (entry.file_id, path)
-
-
-
-# TODO: Maybe a 'mv' command that has the combined move/rename
-# special behaviour of Unix?
-
-def cmd_move(source_list, dest):
-    b = Branch('.')
-
-    b.move([b.relpath(s) for s in source_list], b.relpath(dest))
-
-
-
-def cmd_rename(from_name, to_name):
+    takes_args = ['file+']
+    takes_options = ['verbose']
+    
+    def run(self, file_list, verbose=False):
+        bzrlib.add.smart_add(file_list, verbose)
+
+
+def Relpath(Command):
+    """Show path of a file relative to root"""
+    takes_args = ('filename')
+    
+    def run(self):
+        print Branch(self.args['filename']).relpath(filename)
+
+
+
+class cmd_inventory(Command):
+    """Show inventory of the current working copy or a revision."""
+    takes_options = ['revision']
+    
+    def run(self, revision=None):
+        b = Branch('.')
+        if revision == None:
+            inv = b.read_working_inventory()
+        else:
+            inv = b.get_revision_inventory(b.lookup_revision(revision))
+
+        for path, entry in inv.iter_entries():
+            print '%-50s %s' % (entry.file_id, path)
+
+
+class cmd_move(Command):
+    """Move files to a different directory.
+
+    examples:
+        bzr move *.txt doc
+
+    The destination must be a versioned directory in the same branch.
+    """
+    takes_args = ['source$', 'dest']
+    def run(self, source_list, dest):
+        b = Branch('.')
+
+        b.move([b.relpath(s) for s in source_list], b.relpath(dest))
+
+
+class cmd_rename(Command):
     """Change the name of an entry.
 
-    usage: bzr rename FROM_NAME TO_NAME
-
     examples:
       bzr rename frob.c frobber.c
       bzr rename src/frob.c lib/frob.c
@@ -242,119 +275,127 @@
 
     TODO: Some way to rename multiple files without invoking bzr for each
     one?"""
-    b = Branch('.')
-    b.rename_one(b.relpath(from_name), b.relpath(to_name))
+    takes_args = ['from_name', 'to_name']
     
-
-
-
-def cmd_renames(dir='.'):
+    def run(self, from_name, to_name):
+        b = Branch('.')
+        b.rename_one(b.relpath(from_name), b.relpath(to_name))
+
+
+
+class cmd_renames(Command):
     """Show list of renamed files.
 
-    usage: bzr renames [BRANCH]
-
     TODO: Option to show renames between two historical versions.
 
     TODO: Only show renames under dir, rather than in the whole branch.
     """
-    b = Branch(dir)
-    old_inv = b.basis_tree().inventory
-    new_inv = b.read_working_inventory()
-    
-    renames = list(bzrlib.tree.find_renames(old_inv, new_inv))
-    renames.sort()
-    for old_name, new_name in renames:
-        print "%s => %s" % (old_name, new_name)        
-
-
-
-def cmd_info():
-    """info: Show statistical information for this branch
-
-    usage: bzr info"""
-    import info
-    info.show_info(Branch('.'))        
-    
-
-
-def cmd_remove(file_list, verbose=False):
-    b = Branch(file_list[0])
-    b.remove([b.relpath(f) for f in file_list], verbose=verbose)
-
-
-
-def cmd_file_id(filename):
+    takes_args = ['dir?']
+
+    def run(self, dir='.'):
+        b = Branch(dir)
+        old_inv = b.basis_tree().inventory
+        new_inv = b.read_working_inventory()
+
+        renames = list(bzrlib.tree.find_renames(old_inv, new_inv))
+        renames.sort()
+        for old_name, new_name in renames:
+            print "%s => %s" % (old_name, new_name)        
+
+
+class cmd_info(Command):
+    """Show statistical information for this branch"""
+    def run(self):
+        import info
+        info.show_info(Branch('.'))        
+
+
+class cmd_remove(Command):
+    """Make a file unversioned.
+
+    This makes bzr stop tracking changes to a versioned file.  It does
+    not delete the working copy.
+    """
+    takes_args = ['file+']
+    takes_options = ['verbose']
+    
+    def run(self, file_list, verbose=False):
+        b = Branch(file_list[0])
+        b.remove([b.relpath(f) for f in file_list], verbose=verbose)
+
+
+class cmd_file_id(Command):
     """Print file_id of a particular file or directory.
 
-    usage: bzr file-id FILE
-
     The file_id is assigned when the file is first added and remains the
     same through all revisions where the file exists, even when it is
     moved or renamed.
     """
-    b = Branch(filename)
-    i = b.inventory.path2id(b.relpath(filename))
-    if i == None:
-        bailout("%r is not a versioned file" % filename)
-    else:
-        print i
-
-
-def cmd_file_id_path(filename):
+    hidden = True
+    takes_args = ['filename']
+    def run(self, filename):
+        b = Branch(filename)
+        i = b.inventory.path2id(b.relpath(filename))
+        if i == None:
+            bailout("%r is not a versioned file" % filename)
+        else:
+            print i
+
+
+class cmd_file_path(Command):
     """Print path of file_ids to a file or directory.
 
-    usage: bzr file-id-path FILE
-
     This prints one line for each directory down to the target,
     starting at the branch root."""
-    b = Branch(filename)
-    inv = b.inventory
-    fid = inv.path2id(b.relpath(filename))
-    if fid == None:
-        bailout("%r is not a versioned file" % filename)
-    for fip in inv.get_idpath(fid):
-        print fip
-
-
-def cmd_revision_history():
-    for patchid in Branch('.').revision_history():
-        print patchid
-
-
-def cmd_directories():
-    for name, ie in Branch('.').read_working_inventory().directories():
-        if name == '':
-            print '.'
-        else:
-            print name
-
-
-def cmd_missing():
-    for name, ie in Branch('.').working_tree().missing():
-        print name
-
-
-def cmd_init():
-    # TODO: Check we're not already in a working directory?  At the
-    # moment you'll get an ugly error.
-    
-    # TODO: What if we're in a subdirectory of a branch?  Would like
-    # to allow that, but then the parent may need to understand that
-    # the children have disappeared, or should they be versioned in
-    # both?
-
-    # TODO: Take an argument/option for branch name.
-    Branch('.', init=True)
-
-
-def cmd_diff(revision=None, file_list=None):
-    """bzr diff: Show differences in working tree.
-    
-    usage: bzr diff [-r REV] [FILE...]
-
-    --revision REV
-         Show changes since REV, rather than predecessor.
-
+    hidden = True
+    takes_args = ['filename']
+    def run(self, filename):
+        b = Branch(filename)
+        inv = b.inventory
+        fid = inv.path2id(b.relpath(filename))
+        if fid == None:
+            bailout("%r is not a versioned file" % filename)
+        for fip in inv.get_idpath(fid):
+            print fip
+
+
+class cmd_revision_history(Command):
+    """Display list of revision ids on this branch."""
+    def run(self):
+        for patchid in Branch('.').revision_history():
+            print patchid
+
+
+class cmd_directories(Command):
+    """Display list of versioned directories in this branch."""
+    def run(self):
+        for name, ie in Branch('.').read_working_inventory().directories():
+            if name == '':
+                print '.'
+            else:
+                print name
+
+
+class cmd_init(Command):
+    """Make a directory into a versioned branch.
+
+    Use this to create an empty branch, or before importing an
+    existing project.
+
+    Recipe for importing a tree of files:
+        cd ~/project
+        bzr init
+        bzr add -v .
+        bzr status
+        bzr commit -m 'imported project'
+    """
+    def run(self):
+        Branch('.', init=True)
+
+
+class cmd_diff(Command):
+    """Show differences in working tree.
+    
     If files are listed, only the changes in those files are listed.
     Otherwise, all changes for the tree are listed.
 
@@ -374,313 +415,186 @@
           deleted files.
 
     TODO: This probably handles non-Unix newlines poorly.
-"""
-
-    ## TODO: Shouldn't be in the cmd function.
-
-    b = Branch('.')
-
-    if revision == None:
-        old_tree = b.basis_tree()
-    else:
-        old_tree = b.revision_tree(b.lookup_revision(revision))
-        
-    new_tree = b.working_tree()
-
-    # TODO: Options to control putting on a prefix or suffix, perhaps as a format string
-    old_label = ''
-    new_label = ''
-
-    DEVNULL = '/dev/null'
-    # Windows users, don't panic about this filename -- it is a
-    # special signal to GNU patch that the file should be created or
-    # deleted respectively.
-
-    # TODO: Generation of pseudo-diffs for added/deleted files could
-    # be usefully made into a much faster special case.
-
-    # TODO: Better to return them in sorted order I think.
-
-    if file_list:
-        file_list = [b.relpath(f) for f in file_list]
-
-    # FIXME: If given a file list, compare only those files rather
-    # than comparing everything and then throwing stuff away.
-    
-    for file_state, fid, old_name, new_name, kind in bzrlib.diff_trees(old_tree, new_tree):
-
-        if file_list and (new_name not in file_list):
-            continue
-        
-        # Don't show this by default; maybe do it if an option is passed
-        # idlabel = '      {%s}' % fid
-        idlabel = ''
-
-        # FIXME: Something about the diff format makes patch unhappy
-        # with newly-added files.
-
-        def diffit(oldlines, newlines, **kw):
-            
-            # FIXME: difflib is wrong if there is no trailing newline.
-            # The syntax used by patch seems to be "\ No newline at
-            # end of file" following the last diff line from that
-            # file.  This is not trivial to insert into the
-            # unified_diff output and it might be better to just fix
-            # or replace that function.
-
-            # In the meantime we at least make sure the patch isn't
-            # mangled.
-            
-
-            # Special workaround for Python2.3, where difflib fails if
-            # both sequences are empty.
-            if not oldlines and not newlines:
-                return
-
-            nonl = False
-
-            if oldlines and (oldlines[-1][-1] != '\n'):
-                oldlines[-1] += '\n'
-                nonl = True
-            if newlines and (newlines[-1][-1] != '\n'):
-                newlines[-1] += '\n'
-                nonl = True
-
-            ud = difflib.unified_diff(oldlines, newlines, **kw)
-            sys.stdout.writelines(ud)
-            if nonl:
-                print "\\ No newline at end of file"
-            sys.stdout.write('\n')
-        
-        if file_state in ['.', '?', 'I']:
-            continue
-        elif file_state == 'A':
-            print '*** added %s %r' % (kind, new_name)
-            if kind == 'file':
-                diffit([],
-                       new_tree.get_file(fid).readlines(),
-                       fromfile=DEVNULL,
-                       tofile=new_label + new_name + idlabel)
-        elif file_state == 'D':
-            assert isinstance(old_name, types.StringTypes)
-            print '*** deleted %s %r' % (kind, old_name)
-            if kind == 'file':
-                diffit(old_tree.get_file(fid).readlines(), [],
-                       fromfile=old_label + old_name + idlabel,
-                       tofile=DEVNULL)
-        elif file_state in ['M', 'R']:
-            if file_state == 'M':
-                assert kind == 'file'
-                assert old_name == new_name
-                print '*** modified %s %r' % (kind, new_name)
-            elif file_state == 'R':
-                print '*** renamed %s %r => %r' % (kind, old_name, new_name)
-
-            if kind == 'file':
-                diffit(old_tree.get_file(fid).readlines(),
-                       new_tree.get_file(fid).readlines(),
-                       fromfile=old_label + old_name + idlabel,
-                       tofile=new_label + new_name)
-        else:
-            bailout("can't represent state %s {%s}" % (file_state, fid))
-
-
-
-def cmd_deleted(show_ids=False):
+    """
+    
+    takes_args = ['file*']
+    takes_options = ['revision']
+
+    def run(self, revision=None, file_list=None):
+        from bzrlib.diff import show_diff
+    
+        show_diff(Branch('.'), revision, file_list)
+
+
+class cmd_deleted(Command):
     """List files deleted in the working tree.
 
     TODO: Show files deleted since a previous revision, or between two revisions.
     """
-    b = Branch('.')
-    old = b.basis_tree()
-    new = b.working_tree()
-
-    ## TODO: Much more efficient way to do this: read in new
-    ## directories with readdir, rather than stating each one.  Same
-    ## level of effort but possibly much less IO.  (Or possibly not,
-    ## if the directories are very large...)
-
-    for path, ie in old.inventory.iter_entries():
-        if not new.has_id(ie.file_id):
-            if show_ids:
-                print '%-50s %s' % (path, ie.file_id)
-            else:
-                print path
-
-
-
-def cmd_parse_inventory():
-    import cElementTree
-    
-    cElementTree.ElementTree().parse(file('.bzr/inventory'))
-
-
-
-def cmd_load_inventory():
-    """Load inventory for timing purposes"""
-    Branch('.').basis_tree().inventory
-
-
-def cmd_dump_inventory():
-    Branch('.').read_working_inventory().write_xml(sys.stdout)
-
-
-def cmd_dump_new_inventory():
-    import bzrlib.newinventory
-    inv = Branch('.').basis_tree().inventory
-    bzrlib.newinventory.write_inventory(inv, sys.stdout)
-
-
-def cmd_load_new_inventory():
-    import bzrlib.newinventory
-    bzrlib.newinventory.read_new_inventory(sys.stdin)
-                
-    
-def cmd_dump_slacker_inventory():
-    import bzrlib.newinventory
-    inv = Branch('.').basis_tree().inventory
-    bzrlib.newinventory.write_slacker_inventory(inv, sys.stdout)
-
-
-
-def cmd_dump_text_inventory():
-    import bzrlib.textinv
-    inv = Branch('.').basis_tree().inventory
-    bzrlib.textinv.write_text_inventory(inv, sys.stdout)
-
-
-def cmd_load_text_inventory():
-    import bzrlib.textinv
-    inv = bzrlib.textinv.read_text_inventory(sys.stdin)
-    print 'loaded %d entries' % len(inv)
-    
-    
-
-def cmd_root(filename=None):
-    """Print the branch root."""
-    print bzrlib.branch.find_branch_root(filename)
-    
-
-def cmd_log(timezone='original', verbose=False):
+    def run(self, show_ids=False):
+        b = Branch('.')
+        old = b.basis_tree()
+        new = b.working_tree()
+
+        ## TODO: Much more efficient way to do this: read in new
+        ## directories with readdir, rather than stating each one.  Same
+        ## level of effort but possibly much less IO.  (Or possibly not,
+        ## if the directories are very large...)
+
+        for path, ie in old.inventory.iter_entries():
+            if not new.has_id(ie.file_id):
+                if show_ids:
+                    print '%-50s %s' % (path, ie.file_id)
+                else:
+                    print path
+
+class cmd_root(Command):
+    """Show the tree root directory.
+
+    The root is the nearest enclosing directory with a .bzr control
+    directory."""
+    takes_args = ['filename?']
+    def run(self, filename=None):
+        """Print the branch root."""
+        print bzrlib.branch.find_branch_root(filename)
+
+
+
+class cmd_log(Command):
     """Show log of this branch.
 
-    TODO: Options for utc; to show ids; to limit range; etc.
+    TODO: Options to show ids; to limit range; etc.
     """
-    Branch('.').write_log(show_timezone=timezone, verbose=verbose)
-
-
-def cmd_ls(revision=None, verbose=False):
+    takes_options = ['timezone', 'verbose']
+    def run(self, timezone='original', verbose=False):
+        Branch('.').write_log(show_timezone=timezone, verbose=verbose)
+
+
+class cmd_ls(Command):
     """List files in a tree.
 
     TODO: Take a revision or remote path and list that tree instead.
     """
-    b = Branch('.')
-    if revision == None:
-        tree = b.working_tree()
-    else:
-        tree = b.revision_tree(b.lookup_revision(revision))
-        
-    for fp, fc, kind, fid in tree.list_files():
-        if verbose:
-            if kind == 'directory':
-                kindch = '/'
-            elif kind == 'file':
-                kindch = ''
+    hidden = True
+    def run(self, revision=None, verbose=False):
+        b = Branch('.')
+        if revision == None:
+            tree = b.working_tree()
+        else:
+            tree = b.revision_tree(b.lookup_revision(revision))
+
+        for fp, fc, kind, fid in tree.list_files():
+            if verbose:
+                if kind == 'directory':
+                    kindch = '/'
+                elif kind == 'file':
+                    kindch = ''
+                else:
+                    kindch = '???'
+
+                print '%-8s %s%s' % (fc, fp, kindch)
             else:
-                kindch = '???'
-                
-            print '%-8s %s%s' % (fc, fp, kindch)
-        else:
-            print fp
-    
-    
-
-def cmd_unknowns():
+                print fp
+
+
+
+class cmd_unknowns(Command):
     """List unknown files"""
-    for f in Branch('.').unknowns():
-        print quotefn(f)
-
-
-
-def cmd_ignore(name_pattern):
+    def run(self):
+        for f in Branch('.').unknowns():
+            print quotefn(f)
+
+
+
+class cmd_ignore(Command):
     """Ignore a command or pattern"""
-
-    b = Branch('.')
-
-    # XXX: This will fail if it's a hardlink; should use an AtomicFile class.
-    f = open(b.abspath('.bzrignore'), 'at')
-    f.write(name_pattern + '\n')
-    f.close()
+    takes_args = ['name_pattern']
     
-    inv = b.working_tree().inventory
-    if inv.path2id('.bzrignore'):
-        mutter('.bzrignore is already versioned')
-    else:
-        mutter('need to make new .bzrignore file versioned')
-        b.add(['.bzrignore'])
-
-
-
-def cmd_ignored():
-    """List ignored files and the patterns that matched them.
-      """
-    tree = Branch('.').working_tree()
-    for path, file_class, kind, file_id in tree.list_files():
-        if file_class != 'I':
-            continue
-        ## XXX: Slightly inefficient since this was already calculated
-        pat = tree.is_ignored(path)
-        print '%-50s %s' % (path, pat)
-
-
-def cmd_lookup_revision(revno):
-    try:
-        revno = int(revno)
-    except ValueError:
-        bailout("usage: lookup-revision REVNO",
-                ["REVNO is a non-negative revision number for this branch"])
-
-    print Branch('.').lookup_revision(revno) or NONE_STRING
-
-
-
-def cmd_export(revno, dest):
-    """Export past revision to destination directory."""
-    b = Branch('.')
-    rh = b.lookup_revision(int(revno))
-    t = b.revision_tree(rh)
-    t.export(dest)
-
-def cmd_cat(revision, filename):
-    """Print file to stdout."""
-    b = Branch('.')
-    b.print_file(b.relpath(filename), int(revision))
-
-
-######################################################################
-# internal/test commands
-
-
-def cmd_uuid():
-    """Print a newly-generated UUID."""
-    print bzrlib.osutils.uuid()
-
-
-
-def cmd_local_time_offset():
-    print bzrlib.osutils.local_time_offset()
-
-
-
-def cmd_commit(message=None, verbose=False):
-    """Commit changes to a new revision.
-
-    --message MESSAGE
-        Description of changes in this revision; free form text.
-        It is recommended that the first line be a single-sentence
-        summary.
-    --verbose
-        Show status of changed files,
+    def run(self, name_pattern):
+        b = Branch('.')
+
+        # XXX: This will fail if it's a hardlink; should use an AtomicFile class.
+        f = open(b.abspath('.bzrignore'), 'at')
+        f.write(name_pattern + '\n')
+        f.close()
+
+        inv = b.working_tree().inventory
+        if inv.path2id('.bzrignore'):
+            mutter('.bzrignore is already versioned')
+        else:
+            mutter('need to make new .bzrignore file versioned')
+            b.add(['.bzrignore'])
+
+
+
+class cmd_ignored(Command):
+    """List ignored files and the patterns that matched them."""
+    def run(self):
+        tree = Branch('.').working_tree()
+        for path, file_class, kind, file_id in tree.list_files():
+            if file_class != 'I':
+                continue
+            ## XXX: Slightly inefficient since this was already calculated
+            pat = tree.is_ignored(path)
+            print '%-50s %s' % (path, pat)
+
+
+class cmd_lookup_revision(Command):
+    """Lookup the revision-id from a revision-number
+
+    example:
+        bzr lookup-revision 33
+        """
+    hidden = True
+    def run(self, revno):
+        try:
+            revno = int(revno)
+        except ValueError:
+            raise BzrError("not a valid revision-number: %r" % revno)
+
+        print Branch('.').lookup_revision(revno) or NONE_STRING
+
+
+
+class cmd_export(Command):
+    """Export past revision to destination directory.
+
+    If no revision is specified this exports the last committed revision."""
+    takes_args = ['dest']
+    takes_options = ['revision']
+    def run(self, dest, revno=None):
+        b = Branch('.')
+        if revno == None:
+            rh = b.revision_history[-1]
+        else:
+            rh = b.lookup_revision(int(revno))
+        t = b.revision_tree(rh)
+        t.export(dest)
+
+
+class cmd_cat(Command):
+    """Write a file's text from a previous revision."""
+
+    takes_options = ['revision']
+    takes_args = ['filename']
+
+    def run(self, filename, revision=None):
+        if revision == None:
+            raise BzrCommandError("bzr cat requires a revision number")
+        b = Branch('.')
+        b.print_file(b.relpath(filename), int(revision))
+
+
+class cmd_local_time_offset(Command):
+    """Show the offset in seconds from GMT to local time."""
+    hidden = True    
+    def run(self):
+        print bzrlib.osutils.local_time_offset()
+
+
+
+class cmd_commit(Command):
+    """Commit changes into a new revision.
 
     TODO: Commit only selected files.
 
@@ -688,148 +602,168 @@
 
     TODO: Strict commit that fails if there are unknown or deleted files.
     """
-
-    if not message:
-        bailout("please specify a commit message")
-    Branch('.').commit(message, verbose=verbose)
-
-
-def cmd_check(dir='.'):
-    """check: Consistency check of branch history.
-
-    usage: bzr check [-v] [BRANCH]
-
-    options:
-      --verbose, -v         Show progress of checking.
+    takes_options = ['message', 'verbose']
+    
+    def run(self, message=None, verbose=False):
+        if not message:
+            raise BzrCommandError("please specify a commit message")
+        Branch('.').commit(message, verbose=verbose)
+
+
+class cmd_check(Command):
+    """Validate consistency of branch history.
 
     This command checks various invariants about the branch storage to
     detect data corruption or bzr bugs.
     """
-    import bzrlib.check
-    bzrlib.check.check(Branch(dir, find_root=False))
-
-
-def cmd_is(pred, *rest):
-    """Test whether PREDICATE is true."""
-    try:
-        cmd_handler = globals()['assert_' + pred.replace('-', '_')]
-    except KeyError:
-        bailout("unknown predicate: %s" % quotefn(pred))
-        
-    try:
-        cmd_handler(*rest)
-    except BzrCheckError:
-        # by default we don't print the message so that this can
-        # be used from shell scripts without producing noise
-        sys.exit(1)
-
-
-def cmd_whoami(email=False):
-    """Show bzr user id.
-
-    usage: bzr whoami
-
-    options:
-        --email             Show only the email address.
+    takes_args = ['dir?']
+    def run(self, dir='.'):
+        import bzrlib.check
+        bzrlib.check.check(Branch(dir, find_root=False))
+
+
+
+class cmd_whoami(Command):
+    """Show bzr user id."""
+    takes_options = ['email']
     
-    """
-    if email:
-        print bzrlib.osutils.user_email()
-    else:
-        print bzrlib.osutils.username()
-        
-        
-def cmd_gen_revision_id():
-    print bzrlib.branch._gen_revision_id(time.time())
-
-
-def cmd_selftest():
+    def run(self, email=False):
+        if email:
+            print bzrlib.osutils.user_email()
+        else:
+            print bzrlib.osutils.username()
+
+
+class cmd_selftest(Command):
     """Run internal test suite"""
-    ## -v, if present, is seen by doctest; the argument is just here
-    ## so our parser doesn't complain
-
-    ## TODO: --verbose option
-
-    failures, tests = 0, 0
-    
-    import doctest, bzrlib.store, bzrlib.tests
-    bzrlib.trace.verbose = False
-
-    for m in bzrlib.store, bzrlib.inventory, bzrlib.branch, bzrlib.osutils, \
-        bzrlib.tree, bzrlib.tests, bzrlib.commands, bzrlib.add:
-        mf, mt = doctest.testmod(m)
-        failures += mf
-        tests += mt
-        print '%-40s %3d tests' % (m.__name__, mt),
-        if mf:
-            print '%3d FAILED!' % mf
+    hidden = True
+    def run(self):
+        failures, tests = 0, 0
+
+        import doctest, bzrlib.store, bzrlib.tests
+        bzrlib.trace.verbose = False
+
+        for m in bzrlib.store, bzrlib.inventory, bzrlib.branch, bzrlib.osutils, \
+            bzrlib.tree, bzrlib.tests, bzrlib.commands, bzrlib.add:
+            mf, mt = doctest.testmod(m)
+            failures += mf
+            tests += mt
+            print '%-40s %3d tests' % (m.__name__, mt),
+            if mf:
+                print '%3d FAILED!' % mf
+            else:
+                print
+
+        print '%-40s %3d tests' % ('total', tests),
+        if failures:
+            print '%3d FAILED!' % failures
         else:
             print
 
-    print '%-40s %3d tests' % ('total', tests),
-    if failures:
-        print '%3d FAILED!' % failures
-    else:
-        print
-
-
-
-# deprecated
-cmd_doctest = cmd_selftest
-
-
-######################################################################
-# help
-
-
-def cmd_help(topic=None):
+
+
+class cmd_version(Command):
+    """Show version of bzr"""
+    def run(self):
+        show_version()
+
+def show_version():
+    print "bzr (bazaar-ng) %s" % bzrlib.__version__
+    print bzrlib.__copyright__
+    print "http://bazaar-ng.org/"
+    print
+    print "bzr comes with ABSOLUTELY NO WARRANTY.  bzr is free software, and"
+    print "you may use, modify and redistribute it under the terms of the GNU"
+    print "General Public License version 2 or later."
+
+
+class cmd_rocks(Command):
+    """Statement of optimism."""
+    hidden = True
+    def run(self):
+        print "it sure does!"
+
+
+class cmd_assert_fail(Command):
+    """Test reporting of assertion failures"""
+    hidden = True
+    def run(self):
+        assert False, "always fails"
+
+
+class cmd_help(Command):
+    """Show help on a command or other topic.
+
+    For a list of all available commands, say 'bzr help commands'."""
+    takes_args = ['topic?']
+    
+    def run(self, topic=None):
+        help(topic)
+
+
+def help(topic=None):
     if topic == None:
         print __doc__
     elif topic == 'commands':
         help_commands()
     else:
-        # otherwise, maybe the name of a command?
-        topic, cmdfn = get_cmd_handler(topic)
-
-        doc = getdoc(cmdfn)
-        if doc == None:
-            bailout("sorry, no detailed help yet for %r" % topic)
-
-        print doc
+        help_on_command(topic)
+
+
+def help_on_command(cmdname):
+    cmdname = str(cmdname)
+
+    from inspect import getdoc
+    topic, cmdclass = get_cmd_class(cmdname)
+
+    doc = getdoc(cmdclass)
+    if doc == None:
+        raise NotImplementedError("sorry, no detailed help yet for %r" % cmdname)
+
+    if '\n' in doc:
+        short, rest = doc.split('\n', 1)
+    else:
+        short = doc
+        rest = ''
+
+    print 'usage: bzr ' + topic,
+    for aname in cmdclass.takes_args:
+        aname = aname.upper()
+        if aname[-1] in ['$', '+']:
+            aname = aname[:-1] + '...'
+        elif aname[-1] == '?':
+            aname = '[' + aname[:-1] + ']'
+        elif aname[-1] == '*':
+            aname = '[' + aname[:-1] + '...]'
+        print aname,
+    print 
+    print short
+    if rest:
+        print rest
+
+    if cmdclass.takes_options:
+        print
+        print 'options:'
+        for on in cmdclass.takes_options:
+            print '    --%s' % on
 
 
 def help_commands():
     """List all commands"""
+    import inspect
+    
     accu = []
-    for k in globals().keys():
+    for k, v in globals().items():
         if k.startswith('cmd_'):
-            accu.append(k[4:].replace('_','-'))
+            accu.append((k[4:].replace('_','-'), v))
     accu.sort()
-    print "bzr commands: "
-    for x in accu:
-        print "   " + x
-    print "note: some of these commands are internal-use or obsolete"
-    # TODO: Some kind of marker for internal-use commands?
-    # TODO: Show aliases?
-        
-
-
-
-def cmd_version():
-    print "bzr (bazaar-ng) %s" % bzrlib.__version__
-    print bzrlib.__copyright__
-    print "http://bazaar-ng.org/"
-    print
-    print \
-"""bzr comes with ABSOLUTELY NO WARRANTY.  bzr is free software, and
-you may use, modify and redistribute it under the terms of the GNU 
-General Public License version 2 or later."""
-
-
-def cmd_rocks():
-    """Statement of optimism."""
-    print "it sure does!"
-
+    for cmdname, cmdclass in accu:
+        if cmdclass.hidden:
+            continue
+        print cmdname
+        help = inspect.getdoc(cmdclass)
+        if help:
+            print "    " + help.split('\n', 1)[0]
 
 
 ######################################################################
@@ -858,49 +792,6 @@
     'v':                      'verbose',
 }
 
-# List of options that apply to particular commands; commands not
-# listed take none.
-cmd_options = {
-    'add':                    ['verbose'],
-    'cat':                    ['revision'],
-    'commit':                 ['message', 'verbose'],
-    'deleted':                ['show-ids'],
-    'diff':                   ['revision'],
-    'inventory':              ['revision'],
-    'log':                    ['timezone', 'verbose'],
-    'ls':                     ['revision', 'verbose'],
-    'remove':                 ['verbose'],
-    'status':                 ['all'],
-    'whoami':                 ['email'],
-    }
-
-
-cmd_args = {
-    'add':                    ['file+'],
-    'cat':                    ['filename'],
-    'commit':                 [],
-    'diff':                   ['file*'],
-    'export':                 ['revno', 'dest'],
-    'file-id':                ['filename'],
-    'file-id-path':           ['filename'],
-    'get-file-text':          ['text_id'],
-    'get-inventory':          ['inventory_id'],
-    'get-revision':           ['revision_id'],
-    'get-revision-inventory': ['revision_id'],
-    'help':                   ['topic?'],
-    'ignore':                 ['name_pattern'],
-    'init':                   [],
-    'log':                    [],
-    'lookup-revision':        ['revno'],
-    'move':                   ['source$', 'dest'],
-    'relpath':                ['filename'],
-    'remove':                 ['file+'],
-    'rename':                 ['from_name', 'to_name'],
-    'renames':                ['dir?'],
-    'root':                   ['filename?'],
-    'status':                 [],
-    }
-
 
 def parse_args(argv):
     """Parse command line.
@@ -967,26 +858,12 @@
 
 
 
-def _match_args(cmd, args):
-    """Check non-option arguments match required pattern.
 
-    >>> _match_args('status', ['asdasdsadasd'])
-    Traceback (most recent call last):
-    ...
-    BzrError: ("extra arguments to command status: ['asdasdsadasd']", [])
-    >>> _match_args('add', ['asdasdsadasd'])
-    {'file_list': ['asdasdsadasd']}
-    >>> _match_args('add', 'abc def gj'.split())
-    {'file_list': ['abc', 'def', 'gj']}
-    """
-    # match argument pattern
-    argform = cmd_args.get(cmd, [])
+def _match_argform(cmd, takes_args, args):
     argdict = {}
-    # TODO: Need a way to express 'cp SRC... DEST', where it matches
-    # all but one.
 
-    # step through args and argform, allowing appropriate 0-many matches
-    for ap in argform:
+    # step through args and takes_args, allowing appropriate 0-many matches
+    for ap in takes_args:
         argname = ap[:-1]
         if ap[-1] == '?':
             if args:
@@ -999,14 +876,14 @@
                 argdict[argname + '_list'] = None
         elif ap[-1] == '+':
             if not args:
-                bailout("command %r needs one or more %s"
+                raise BzrCommandError("command %r needs one or more %s"
                         % (cmd, argname.upper()))
             else:
                 argdict[argname + '_list'] = args[:]
                 args = []
         elif ap[-1] == '$': # all but one
             if len(args) < 2:
-                bailout("command %r needs one or more %s"
+                raise BzrCommandError("command %r needs one or more %s"
                         % (cmd, argname.upper()))
             argdict[argname + '_list'] = args[:-1]
             args[:-1] = []                
@@ -1014,14 +891,14 @@
             # just a plain arg
             argname = ap
             if not args:
-                bailout("command %r requires argument %s"
+                raise BzrCommandError("command %r requires argument %s"
                         % (cmd, argname.upper()))
             else:
                 argdict[argname] = args.pop(0)
             
     if args:
-        bailout("extra arguments to command %s: %r"
-                % (cmd, args))
+        raise BzrCommandError("extra argument to command %s: %s"
+                              % (cmd, args[0]))
 
     return argdict
 
@@ -1039,15 +916,13 @@
     try:
         args, opts = parse_args(argv[1:])
         if 'help' in opts:
-            # TODO: pass down other arguments in case they asked for
-            # help on a command name?
             if args:
-                cmd_help(args[0])
+                help(args[0])
             else:
-                cmd_help()
+                help()
             return 0
         elif 'version' in opts:
-            cmd_version()
+            cmd_version([], [])
             return 0
         cmd = str(args.pop(0))
     except IndexError:
@@ -1055,7 +930,7 @@
         log_error('  try "bzr help"')
         return 1
 
-    canonical_cmd, cmd_handler = get_cmd_handler(cmd)
+    canonical_cmd, cmd_class = get_cmd_class(cmd)
 
     # global option
     if 'profile' in opts:
@@ -1065,27 +940,24 @@
         profile = False
 
     # check options are reasonable
-    allowed = cmd_options.get(canonical_cmd, [])
+    allowed = cmd_class.takes_options
     for oname in opts:
         if oname not in allowed:
-            bailout("option %r is not allowed for command %r"
-                    % (oname, cmd))
-
-    # TODO: give an error if there are any mandatory options which are
-    # not specified?  Or maybe there shouldn't be any "mandatory
-    # options" (it is an oxymoron)
+            raise BzrCommandError("option %r is not allowed for command %r"
+                                  % (oname, cmd))
 
     # mix arguments and options into one dictionary
-    cmdargs = _match_args(canonical_cmd, args)
+    cmdargs = _match_argform(cmd, cmd_class.takes_args, args)
+    cmdopts = {}
     for k, v in opts.items():
-        cmdargs[k.replace('-', '_')] = v
+        cmdopts[k.replace('-', '_')] = v
 
     if profile:
         import hotshot
         pffileno, pfname = tempfile.mkstemp()
         try:
             prof = hotshot.Profile(pfname)
-            ret = prof.runcall(cmd_handler, **cmdargs) or 0
+            ret = prof.runcall(cmd_class, cmdopts, cmdargs) or 0
             prof.close()
 
             import hotshot.stats
@@ -1102,7 +974,7 @@
             os.close(pffileno)
             os.remove(pfname)
     else:
-        return cmd_handler(**cmdargs) or 0
+        cmdobj = cmd_class(cmdopts, cmdargs) or 0
 
 
 
@@ -1119,9 +991,6 @@
         sys.stderr.write('  see ~/.bzr.log for debug information\n')
 
 
-def cmd_assert_fail():
-    assert False, "always fails"
-
 
 def main(argv):
     import errno
@@ -1135,7 +1004,8 @@
             sys.stdout.flush()
             return ret
         except BzrError, e:
-            _report_exception(e, 'error: ' + e.args[0])
+            quiet = isinstance(e, (BzrCommandError))
+            _report_exception(e, 'error: ' + e.args[0], quiet=quiet)
             if len(e.args) > 1:
                 for h in e.args[1]:
                     # some explanation or hints

=== modified file 'bzrlib/diff.py'
--- bzrlib/diff.py	2005-04-06 03:38:35 +0000
+++ bzrlib/diff.py	2005-05-03 07:48:16 +0000
@@ -21,9 +21,6 @@
 
 
 
-
-
-
 def diff_trees(old_tree, new_tree):
     """Compute diff between two trees.
 
@@ -142,3 +139,112 @@
             old_item = next(old_it)
 
 
+
+def show_diff(b, revision, file_list):
+    import difflib, sys
+    
+    if revision == None:
+        old_tree = b.basis_tree()
+    else:
+        old_tree = b.revision_tree(b.lookup_revision(revision))
+        
+    new_tree = b.working_tree()
+
+    # TODO: Options to control putting on a prefix or suffix, perhaps as a format string
+    old_label = ''
+    new_label = ''
+
+    DEVNULL = '/dev/null'
+    # Windows users, don't panic about this filename -- it is a
+    # special signal to GNU patch that the file should be created or
+    # deleted respectively.
+
+    # TODO: Generation of pseudo-diffs for added/deleted files could
+    # be usefully made into a much faster special case.
+
+    # TODO: Better to return them in sorted order I think.
+
+    if file_list:
+        file_list = [b.relpath(f) for f in file_list]
+
+    # FIXME: If given a file list, compare only those files rather
+    # than comparing everything and then throwing stuff away.
+    
+    for file_state, fid, old_name, new_name, kind in diff_trees(old_tree, new_tree):
+
+        if file_list and (new_name not in file_list):
+            continue
+        
+        # Don't show this by default; maybe do it if an option is passed
+        # idlabel = '      {%s}' % fid
+        idlabel = ''
+
+        # FIXME: Something about the diff format makes patch unhappy
+        # with newly-added files.
+
+        def diffit(oldlines, newlines, **kw):
+            
+            # FIXME: difflib is wrong if there is no trailing newline.
+            # The syntax used by patch seems to be "\ No newline at
+            # end of file" following the last diff line from that
+            # file.  This is not trivial to insert into the
+            # unified_diff output and it might be better to just fix
+            # or replace that function.
+
+            # In the meantime we at least make sure the patch isn't
+            # mangled.
+            
+
+            # Special workaround for Python2.3, where difflib fails if
+            # both sequences are empty.
+            if not oldlines and not newlines:
+                return
+
+            nonl = False
+
+            if oldlines and (oldlines[-1][-1] != '\n'):
+                oldlines[-1] += '\n'
+                nonl = True
+            if newlines and (newlines[-1][-1] != '\n'):
+                newlines[-1] += '\n'
+                nonl = True
+
+            ud = difflib.unified_diff(oldlines, newlines, **kw)
+            sys.stdout.writelines(ud)
+            if nonl:
+                print "\\ No newline at end of file"
+            sys.stdout.write('\n')
+        
+        if file_state in ['.', '?', 'I']:
+            continue
+        elif file_state == 'A':
+            print '*** added %s %r' % (kind, new_name)
+            if kind == 'file':
+                diffit([],
+                       new_tree.get_file(fid).readlines(),
+                       fromfile=DEVNULL,
+                       tofile=new_label + new_name + idlabel)
+        elif file_state == 'D':
+            assert isinstance(old_name, types.StringTypes)
+            print '*** deleted %s %r' % (kind, old_name)
+            if kind == 'file':
+                diffit(old_tree.get_file(fid).readlines(), [],
+                       fromfile=old_label + old_name + idlabel,
+                       tofile=DEVNULL)
+        elif file_state in ['M', 'R']:
+            if file_state == 'M':
+                assert kind == 'file'
+                assert old_name == new_name
+                print '*** modified %s %r' % (kind, new_name)
+            elif file_state == 'R':
+                print '*** renamed %s %r => %r' % (kind, old_name, new_name)
+
+            if kind == 'file':
+                diffit(old_tree.get_file(fid).readlines(),
+                       new_tree.get_file(fid).readlines(),
+                       fromfile=old_label + old_name + idlabel,
+                       tofile=new_label + new_name)
+        else:
+            bailout("can't represent state %s {%s}" % (file_state, fid))
+
+

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2005-03-09 04:08:15 +0000
+++ bzrlib/errors.py	2005-05-03 07:48:16 +0000
@@ -29,6 +29,9 @@
     pass
 
 
+class BzrCommandError(BzrError):
+    # Error from malformed user command
+    pass
 
 
 def bailout(msg, explanation=[]):

=== modified file 'bzrlib/textui.py'
--- bzrlib/textui.py	2005-04-15 07:53:59 +0000
+++ bzrlib/textui.py	2005-05-03 07:48:16 +0000
@@ -22,7 +22,7 @@
         # use this even on windows?
         kind_ch = '/'
     else:
-        assert kind == 'file'
+        assert kind == 'file', ("can't handle file of type %r" % kind)
         kind_ch = ''
 
     assert len(state) == 1

=== modified file 'doc/revfile.txt'
--- doc/revfile.txt	2005-05-03 02:39:45 +0000
+++ doc/revfile.txt	2005-05-03 07:48:16 +0000
@@ -268,3 +268,4 @@
   that full-texts cannot just simply be copied out but rather composed
   from chunks.  That seems inefficient since it is probably common to
   only want the text.
+

